﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApplication1
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        const int numPoints = 5;
        double[,] xy = new double[numPoints, 2];
        double[][] d = new double[numPoints][];
        Ellipse[] ellipses = new Ellipse[numPoints];
        Brush[] brushes = new Brush[5];
        Random random = new Random();

        System.Windows.Threading.DispatcherTimer timer = new System.Windows.Threading.DispatcherTimer();

        public MainWindow()
        {
            InitializeComponent();

            for (int i = 0; i < numPoints; i++)
            {
                d[i] = new double[i];
                for (int j = 0; j < i; j++)
			    {
                    d[i][j] = 100.0*random.NextDouble();
			    }

                xy[i, 0] = random.NextDouble() + 0.5 * Width;
                xy[i, 1] = random.NextDouble() + 0.5 * Height;
            }

            brushes[0] = Brushes.Red;
            brushes[1] = Brushes.Green;
            brushes[2] = Brushes.Blue;
            brushes[3] = Brushes.Brown;
            brushes[4] = Brushes.Cyan;

            for (int i = 0; i < numPoints; i++)
            {
                Ellipse myEllipse = new Ellipse();
                myEllipse.Fill = brushes[i % brushes.Length];
                myEllipse.Width = 5.0;
                myEllipse.Height = 5.0;
                myCanvas.Children.Add(myEllipse);
                ellipses[i] = myEllipse;
            }

            timer.Interval = TimeSpan.FromSeconds(0.1);
            timer.Tick += layout;
            timer.IsEnabled = true;
        }

        void layout(object sender, EventArgs e)
        {
            double misfit = 0.0;
            for (int i = 0; i < numPoints; i++)
            {
                for (int j = 0; j < i; j++)
                {
                    double dx = xy[i, 0] - xy[j, 0];
                    double dy = xy[i, 1] - xy[j, 1];
                    double dist = Math.Sqrt(dx*dx + dy*dy) + 0.01;

                    // TODO: Zusammenschieben, wenn diff < 0.0; langsamer schieben, wenn Anzahl groß
                    double diff = d[i][j] - dist;
                    if(diff > 0.0)
                    {
                        xy[i, 0] += dx / dist;
                        xy[i, 1] += dy / dist;
                        xy[j, 0] -= dx / dist;
                        xy[j, 1] -= dy / dist;
                    }

                    misfit += Math.Abs(diff);
                }
            }

            Title = "" + misfit;

            for (int i = 0; i < numPoints; i++)
            {
                Canvas.SetLeft(ellipses[i], xy[i, 0]);
                Canvas.SetTop(ellipses[i], xy[i, 1]);
            }
        }
    }
}
